@@ -9,9 +9,10 @@ from shutil import copyfile, copymode, move
 from tempfile import mkstemp
 from typing import Dict, Final, List, Tuple, Union
 
-from Crypto.PublicKey import RSA
-from ec2instances.ec2_instance_mapping import Ec2RemoteShellMapping
-from sshconf import empty_ssh_config_file, read_ssh_config
+from cryptography.hazmat.backends import default_backend as crypto_default_backend
+from cryptography.hazmat.primitives import serialization as crypto_serialization
+from cryptography.hazmat.primitives.asymmetric import rsa
+from ec2instances.ec2_instance_mapping import Ec2RemoteShellMapping, Ec2RemoteShellProxy
 
 from pyclvm._common.azure_instance_mapping import (
     AzureRemoteShellMapping,
@@ -38,16 +39,123 @@ _GOOGLE_SSH_PUB_KEY: Final[str] = f"{_SSH_DIR}/{_GOOGLE_SSH_PRIV_KEY_NAME}.pub"
 _GOOGLE_SSH_KNOWN_HOSTS: Final[str] = f"{_SSH_DIR}/google_compute_known_hosts"
 
 _BACKUP_SUFFIX = "ORIG"
+_MAIN = sys.modules["__main__"].__file__
+_PLATFORM = _default_platform().lower()
 
 
+def new(instance_name: str, **kwargs: str) -> Union[Dict, None]:
+    """
+    create new ssh key for particular Virtual Machine
+
+    Args:
+        instance_name (str): Virtual Machine instance name
+        **kwargs (str): (optional) classifiers, at the moment, profile name, also look at shell
+
+    Returns:
+        None
+    """
+    default_platform, supported_platforms = (
+        _default_platform(**kwargs),
+        _get_supported_platforms(),
+    )
+
+    if default_platform in supported_platforms:
+        return {
+            "AWS": partial(_new_aws, instance_name, **kwargs),
+            "GCP": partial(_new_gcp, instance_name, **kwargs),
+            "AZURE": partial(_new_azure, instance_name, **kwargs),
+        }[default_platform.upper()]()
+    else:
+        _unsupported_platform(default_platform)
+
+
+# ---
+def _new_aws(instance_name: str, **kwargs: str) -> None:
+    instance = Ec2RemoteShellMapping(get_session(kwargs)).get(instance_name)
+    _create_config_block(instance=instance, **kwargs)
+
+
+# ---
+def _new_gcp(instance_name: str, **kwargs: str) -> None:
+    instance = GcpRemoteShellMapping().get(instance_name)
+    _create_config_block(instance=instance, **kwargs)
+
+
+# ---
+def _new_azure(instance_name: str, **kwargs: str) -> None:
+    instance = AzureRemoteShellMapping().get(instance_name)
+    _create_config_block(instance=instance, **kwargs)
+
+
+# ---
+def _aws_config_lines(instance: Ec2RemoteShellProxy, **kwargs: str):
+    profile = kwargs.get("profile", "default")
+    private_key_name, pubkey = _save_keys(profile, instance.name)
+
+    proxy_data = {
+        "identity_file": private_key_name,
+        "proxy_command": f"{str(_MAIN)} ssh start {instance.name} %p profile={profile} platform={_PLATFORM}",
+        "user_name": "ssm-user",
+    }
+    instance.execute(
+        "pip3 install --upgrade authk",
+        f'runuser -u ssm-user -- authk add "{pubkey}"',
+        **kwargs,
+    )
+    return _config_lines(instance.name, proxy_data)
+
+
+# ---
+def _gcp_config_lines(instance: GcpRemoteShellProxy, **kwargs: str):
+    return _config_lines(
+        instance.name, _get_gcp_proxy_data(instance=instance, **kwargs)
+    )
+
+
+# ---
+def _azure_config_lines(instance: AzureRemoteShellProxy, **kwargs: str) -> List:
+    profile = kwargs.get("profile", "default")
+
+    account = kwargs.get("account")
+    key = kwargs.get("key")
+    if not account or not key:
+        print(
+            "\n------------\nSpecify account=account_name or/and key=/path/to/ssh/key/file\n"
+            'e.g "clvm ssh new vm-instance-name account=username '
+            'key=/path/to/ssh/key/file platform=azure"\n'
+        )
+        sys.exit(-1)
+
+    port = next_free_port(port=22060, max_port=22960)
+    proxy_data = {
+        "identity_file": key,
+        "proxy_command": f"{str(_MAIN)} ssh start {instance.name} {port} profile={profile} platform={_PLATFORM}",
+        "user_name": account,
+    }
+    return _config_lines(instance.name, proxy_data)
+
+
+# ---------------------------------
+# --- Generate new SSH key pair ---
 def _format_public_key(pubkey) -> str:
-    return f'{pubkey.exportKey(r"OpenSSH").decode("utf-8")} {getpass.getuser()}@{platform.node()}'
+    return f'{pubkey.decode("utf-8")} {getpass.getuser()}@{platform.node()}'
 
 
 def _generate_keys() -> Tuple[str, str]:
-    key = RSA.generate(1024)
-    pubkey = key.publickey()
-    return key.exportKey("PEM").decode("utf-8"), _format_public_key(pubkey)
+    key = rsa.generate_private_key(
+        backend=crypto_default_backend(), public_exponent=65537, key_size=2048
+    )
+
+    private_key = key.private_bytes(
+        crypto_serialization.Encoding.PEM,
+        crypto_serialization.PrivateFormat.PKCS8,
+        crypto_serialization.NoEncryption(),
+    )
+
+    public_key = key.public_key().public_bytes(
+        crypto_serialization.Encoding.OpenSSH, crypto_serialization.PublicFormat.OpenSSH
+    )
+    return private_key.decode("utf-8"), _format_public_key(public_key)
 
 
 def _save_keys(profile: str, instance_name: str) -> Tuple[str, str]:
@@ -71,117 +179,12 @@ def _save_keys(profile: str, instance_name: str) -> Tuple[str, str]:
     return private_key_name, pubkey
 
 
-def _update_ssh_config(
-    instance_name: str, private_key_name: str, profile: str, platform: str
-) -> None:
-    _main = sys.modules["__main__"].__file__
-    try:
-        conf = read_ssh_config(_SSH_CONFIG)
-    except FileNotFoundError:
-        conf = empty_ssh_config_file()
-    params = {
-        "IdentityFile": private_key_name,
-        "ProxyCommand": f"{str(_main)} ssh start %h %p profile={profile} platform={platform}",
-        "User": "ssm-user",
-    }
-    func = conf.set if conf.host(instance_name) else conf.add
-    func(instance_name, **params)
-    conf.write(_SSH_CONFIG)
+# --- End of Generate new SSH key pair ---
+# ----------------------------------------
 
 
-def new(instance_name: str, **kwargs: str) -> Union[Dict, None]:
-    """
-    create new ssh key for particular Virtual Machine
-
-    Args:
-        instance_name (str): Virtual Machine instance name
-        **kwargs (str): (optional) classifiers, at the moment, profile name, also look at shell
-
-    Returns:
-        None
-    """
-    default_platform, supported_platforms = (
-        _default_platform(**kwargs),
-        _get_supported_platforms(),
-    )
-
-    if default_platform in supported_platforms:
-        return {
-            "AWS": partial(_new_aws, instance_name, default_platform.lower(), **kwargs),
-            "GCP": partial(_new_gcp, instance_name, **kwargs),
-            "AZURE": partial(_new_azure, instance_name, **kwargs),
-        }[default_platform.upper()]()
-    else:
-        _unsupported_platform(default_platform)
-
-
-# ---
-def _new_aws(instance_name: str, platform_: str, **kwargs: str) -> None:
-    instance = Ec2RemoteShellMapping(get_session(kwargs))[instance_name]
-    profile = kwargs.get("profile", "default")
-    private_key_name, pubkey = _save_keys(profile, instance_name)
-    _update_ssh_config(instance_name, private_key_name, profile, platform_)
-    instance.execute(
-        "pip3 install --upgrade authk",
-        f'runuser -u ssm-user -- authk add "{pubkey}"',
-        **kwargs,
-    )
-
-
-# ---
-def _new_gcp(instance_name: str, **kwargs: str) -> None:
-    instance = GcpRemoteShellMapping().get(instance_name)
-    _create(instance_name=instance_name, instance=instance, **kwargs)
-
-
-# ---
-def _new_azure(instance_name: str, **kwargs: str) -> None:
-    instance = AzureRemoteShellMapping().get(instance_name)
-    _create(instance_name=instance_name, instance=instance, **kwargs)
-
-
-# ----------------------
-# ----------------------
-# ----------------------
-def _get_gcp_proxy_data(instance: GcpRemoteShellProxy, **kwargs: str) -> Dict:
-    """
-    Gets proxy string and real GCP instance's username
-    Args:
-        instance (str): The name of an instance to run
-        username (str): The name of a user to log in to the instance
-        **kwargs (str): (optional) additional arguments
-    Returns:
-        (jdict) Retrieved data
-    """
-
-    kwargs["dry_run"] = "yes"
-    kwargs["capture_output"] = True
-    _stdout = instance.execute((), **kwargs).stdout.decode("utf8")
-    try:
-        ind_1 = _stdout.index("ProxyCommand") + 13
-        ind_2 = _stdout.index('" -o ProxyUseFdpass=no')
-
-        account = kwargs.get("account")
-        if not account:
-            raise ValueError(
-                "No username/account, use [--account=username@company.com]"
-            )
-        for c in ("@", ".", "-"):
-            account = account.replace(c, "_")
-
-        return {
-            "identity_file": _GOOGLE_SSH_PRIV_KEY,
-            "proxy_command": _stdout[ind_1:ind_2],
-            "user_name": account[:33].strip("_"),
-        }
-    except ValueError as err:
-        raise RuntimeError(
-            "\n------------\nNo such VM name or/and account. Set the existing VM name and account.\n"
-            'e.g "clvm ssh new vm-instance-name account=username@domain.com platform=gcp"\n'
-        ) from err
-
-
-# ---
+# ----------------------------------------
+# --- Analysis and writing config file ---
 def _analyze_config(instance_name: str, _platform: str) -> Tuple[int, int]:
     previous_conf_pos = -1
     wildcard_conf_pos = -1
@@ -197,10 +200,9 @@ def _analyze_config(instance_name: str, _platform: str) -> Tuple[int, int]:
 
 
 # ---
-def _config_lines(instance_name: str, proxy_data: Dict, _platform: str) -> List:
+def _config_lines(instance_name: str, proxy_data: Dict) -> List:
     lines = [
-        f"Host {instance_name}-{_platform}\n",
-        f"  HostName {instance_name}\n",
+        f"Host {instance_name}-{_PLATFORM}\n",
         f"  IdentityFile {proxy_data['identity_file']}\n",
         f"  ProxyCommand {proxy_data['proxy_command']}\n",
         f"  User {proxy_data['user_name']}\n",
@@ -285,9 +287,8 @@ def _update_config(instance_name: str, config_lines: List, _platform: str) -> No
 
 
 # ---
-def _create(
-    instance_name: str,
-    instance: Union[GcpRemoteShellProxy, AzureRemoteShellProxy],
+def _create_config_block(
+    instance: Union[GcpRemoteShellProxy, AzureRemoteShellProxy, Ec2RemoteShellProxy],
     **kwargs: str,
 ) -> None:
     """
@@ -300,23 +301,18 @@ def _create(
     Returns:
         None
     """
-    _platform = ""
-    config_lines = []
-    if isinstance(instance, GcpRemoteShellProxy):
-        _platform = "gcp"
-        config_lines = _config_lines(
-            instance_name, _get_gcp_proxy_data(instance=instance, **kwargs), _platform
-        )
-    elif isinstance(instance, AzureRemoteShellProxy):
-        _platform = "azure"
-        config_lines = _azure_config_lines(instance_name, **kwargs)
+    config_lines = {
+        "aws": partial(_aws_config_lines, instance, **kwargs),
+        "gcp": partial(_gcp_config_lines, instance, **kwargs),
+        "azure": partial(_azure_config_lines, instance, **kwargs),
+    }[_PLATFORM]()
 
     os.makedirs(name=_SSH_DIR, mode=0o700, exist_ok=True)
 
     if os.path.isfile(_SSH_CONFIG):
         _backup(_SSH_CONFIG, 0o600)
         _update_config(
-            instance_name=instance_name, config_lines=config_lines, _platform=_platform
+            instance_name=instance.name, config_lines=config_lines, _platform=_PLATFORM
         )
     else:
         with open(
@@ -325,24 +321,54 @@ def _create(
             [config_file.write(line) for line in config_lines]
 
 
-# ---
-def _azure_config_lines(instance_name: str, **kwargs: str) -> List:
-    instance = AzureRemoteShellMapping().get(instance_name)
+# --- End of Analysis and writing config file ---
+# -----------------------------------------------
 
-    _account = kwargs.get("account")
-    _key = kwargs.get("key")
-    if not _account or not _key:
-        raise RuntimeError(
-            "\n------------\nSpecify account=account_name or/and key=/path/to/ssh/key/file\n"
-            'e.g "clvm ssh new vm-instance-name account=username '
-            'key=/path/to/ssh/key/file platform=azure"\n'
+
+# -------------------------------------------------
+# --- Special for Google SDK extract proxy data ---
+def _get_gcp_proxy_data(instance: GcpRemoteShellProxy, **kwargs: str) -> Dict:
+    """
+    Gets proxy string and real GCP instance's username
+    Args:
+        instance (str): The name of an instance to run
+        username (str): The name of a user to log in to the instance
+        **kwargs (str): (optional) additional arguments
+    Returns:
+        (jdict) Retrieved data
+    """
+    profile = kwargs.get("profile", "default")
+    kwargs["dry_run"] = "yes"
+    kwargs["capture_output"] = True
+    _stdout = instance.execute((), **kwargs).stdout.decode("utf8")
+    try:
+        _stdout.index("ProxyCommand") + 13
+        # ind_2 = _stdout.index('" -o ProxyUseFdpass=no')
+
+        account = kwargs.get("account")
+        if not account:
+            raise ValueError(
+                "No username/account, use [--account=username@company.com]"
+            )
+        for c in ("@", ".", "-"):
+            account = account.replace(c, "_")
+
+        return {
+            "identity_file": _GOOGLE_SSH_PRIV_KEY,
+            # "proxy_command": _stdout[ind_1:ind_2],
+            "proxy_command": f"{str(_MAIN)} ssh start {instance.name} %p profile={profile} platform={_PLATFORM}",
+            "user_name": account[:32].strip(
+                "_"
+            ),  # Google OSLogin account name length is <= 32 without
+            # underline sign at the end
+        }
+    except ValueError:
+        print(
+            "\n------------\nNo such VM name or/and account. Set the existing VM name or/and account.\n"
+            'e.g "clvm ssh new vm-instance-name account=username@domain.com platform=gcp"\n'
         )
+        sys.exit(-1)
 
-    _port = next_free_port(port=22060, max_port=22160)
-    instance_name = instance.name
-    proxy_data = {
-        "identity_file": _key,
-        "proxy_command": f"clvm ssh start {instance_name} {_port} platform=azure",
-        "user_name": _account,
-    }
-    return _config_lines(instance_name, proxy_data, "azure")
+
+# --- End of Special for Google SDK extract proxy data ---
+# --------------------------------------------------------
